#include "config.h"

#include "abc-utils.h"

__attribute__((__format__(printf, 1, 2))) void verbose(const char *format, ...)
{
	if (opt_verbose) {
		va_list args;
		va_start(args, format);
		vprintf(format, args);
		va_end(args);
	}
}

__attribute__((__format__(printf, 1, 2))) void error(const char *format, ...)
{
	va_list args;

	va_start(args, format);
	vfprintf(stderr, format, args);
	va_end(args);
}

__attribute__((__noreturn__)) __attribute__((__format__(printf, 1, 2))) void
fatal(const char *format, ...)
{
	va_list args;

	va_start(args, format);
	vfprintf(stderr, format, args);
	va_end(args);

	exit(1);
}

__attribute__((__noreturn__)) void oom()
{
	fatal("Out of memory");
}

bool str_has_prefix(const char *str, const char *prefix)
{
	return strncmp(str, prefix, strlen(prefix)) == 0;
}

bool str_has_suffix(const char *str, const char *suffix)
{
	size_t str_len;
	size_t suffix_len;

	str_len = strlen(str);
	suffix_len = strlen(suffix);

	if (str_len < suffix_len)
		return false;

	return strcmp(str + str_len - suffix_len, suffix) == 0;
}

char *join_path(const char *a, const char *b)
{
	size_t a_len = strlen(a);
	size_t b_len = strlen(b);

	bool add_slash = a_len > 0 && a[a_len - 1] != '/';

	size_t len = a_len + b_len + 1;
	if (add_slash)
		len++;

	char *res = malloc(len);
	if (res) {
		size_t res_size = a_len;
		memcpy(res, a, a_len);

		if (add_slash) {
			memcpy(res + res_size, "/", 1);
			res_size += 1;
		}
		memcpy(res + res_size, b, b_len + 1);
	}
	return res;
}

char *read_file(const char *filename)
{
	autoclose int fd = -1;
	size_t n_read;
	size_t buf_size;

	fd = open(filename, O_RDONLY);
	if (fd == -1)
		return NULL;

	buf_size = 4096;
	autofree char *buf = malloc(buf_size);
	if (buf == NULL)
		oom();

	n_read = 0;
	while (1) {
		if (n_read == buf_size) {
			buf_size *= 2;
			buf = realloc(buf, buf_size);
			if (buf == NULL)
				oom();
		}

		ssize_t r = read(fd, buf + n_read, buf_size - n_read);
		if (r < 0)
			return NULL;

		n_read += r;
		if (r == 0) /* early eof */
			break;
	}

	// Always null-terminate
	buf[n_read] = 0;

	return steal_ptr(&buf);
}

/* Note: Modifies row */
char **str_split(char *row, char split_at, size_t *size)
{
	size_t n;
	char **res;

	n = 1;
	for (int i = 0; row[i] != 0; i++) {
		if (row[i] == split_at)
			n++;
	}

	res = calloc(n + 1, sizeof(char *));
	if (res == NULL)
		oom();

	n = 0;
	res[n++] = row;
	while (*row != 0) {
		while (*row != 0 && *row != split_at)
			row++;
		if (*row == split_at) {
			*row = 0;
			row++;
			res[n++] = row;
		}
	}

	if (size)
		*size = n;

	res[n] = NULL; /* NULL terminate */

	return res;
}
